
#
# confcap - 
#	Conference Presentation Capture Application (v1.0 May 2005)
#	Larry Rowe

source confcap.cfg
source ncast.tcl
source kramer.tcl
source menubar.tcl
source utils.tcl
source button-icons.tcl

# debug with tkcon visible
# turn off tcl-expand of the following
\source c:/tcl/bin/tkcon.tcl
package require tkcon
tkcon show


#
# Global Variables
#
set CPC(tkConsole) 1	;# default show, for debugging
set CPC(state) "READY"	;# READY -> RECORDING -> {PAUSE -> RECORDING}* -> STOP
set CPC(stateWidget) {}	;# entry widget where state is displayed
set CPC(timecodeWidget) {}	;# entry widget where timecode is displayed
set CPC(recordingTime) "00:00:00"
set CPC(recordBase) 0
set CPC(recordStart) 0
set CPC(recordCallback) {}
set CPC(lfd) 0
# NCast G2 connection
# CPC(g2Host) - set in confcap.cfg
# CPC(g2Port) - set in confcap.cfg
set CPC(g2Sock) 0
set CPC(g2Busy) 0
set CPC(g2Loopback) 0
set CPC(g2LoopbackWidget) 0
# Kramer VP720-DS state
# CPC(vpDevice) - set in confcap.cfg
set CPC(vpCHID) ""
set CPC(vpSource) "Unknown"
set CPC(vpLastVideo) "Unknown"
set CPC(vpPip) 0
set CPC(vpBlack) 0
set CPC(vpTimeout) 600
set CPC(vpOutput) ""
set CPC(vpMode) ""
# VCC3 configuration
# CPC(vccDevice) - set in confcap.cfg
# CPC(vccServerHost) - set in confcap.cfg
# CPC(vccServerPort) - set in confcap.cfg

# Toggle Tkconsole window
proc toggleTkConsole {} {
	global CPC

	if {$CPC(tkConsole)} {
		tkcon show
	} else {
		tkcon hide
	}
}
#
# Procedure to track recording time
# Invoked once per second
#
proc updateTimeCode {} {
	global CPC

	set CPC(recordingTime) [getCurrentTimeCode]
	set CPC(recordCallback) [after 1000 updateTimeCode]
}

proc getCurrentTimeCode {} {
	global CPC

	return [convertTime 	\
		[expr $CPC(recordBase)+([clock seconds]-$CPC(recordStart))]]
}

#
# Procedures to execute NCast G2 operations
#
proc openG2 {} {
	global CPC G2

	if {$G2(State) != "CLOSED"} {
		puts stderr "openG2: inconsistent interface, G2(State)!=\"CLOSED\""
		return
	} 
	if {[catch {NCast_Open $CPC(g2Host) $CPC(g2Port)} retval] != 0} {
		tk_messageBox -type ok -message "G2: $retval"
	} else {
		set CPC(g2Sock) $retval
		puts stderr "G2 openned"
	}
}

proc closeG2 {} {
	global CPC

	catch {NCast_Close $CPC(g2Sock)}
	set CPC(g2Sock) ""
}

#
# Procedures to start/pause/stop/reset recording
#
proc startRecording { w } {
	global CPC

	set buttons .gui.top.record
	switch -exact -- $CPC(state) { 
		"READY" {
			set CPC(recordStart) [clock seconds]
			NCast_RecordOn $CPC(g2Sock)
			writeLog $CPC(lfd) "$CPC(talkNumber): start $CPC(recordingTime) ([clock format [clock seconds]])"
		} 
		"PAUSE" {
			set CPC(recordStart) [clock seconds]
			NCast_RecordContinue $CPC(g2Sock)
			writeLog $CPC(lfd) "$CPC(talkNumber): resume $CPC(recordingTime)"
		} 
		default {
			puts stderr "confcap startRecording: interface error"
		}
	}

	set CPC(state) "RECORDING"
	$CPC(stateWidget) configure -fg red
	$CPC(timecodeWidget) configure -fg red
	$w.start configure -image $CPC(pauseImage)\
   		-command " pauseRecording $w " 
	undim $w.stop
	set CPC(recordCallback) [after 1000 updateTimeCode]
}

proc pauseRecording { w } {
	global CPC

	after cancel $CPC(recordCallback)
	set CPC(recordCallback) {}
	NCast_RecordPause $CPC(g2Sock)
	incr CPC(recordBase) [expr [clock seconds]-$CPC(recordStart)]
	writeLog $CPC(lfd) "$CPC(talkNumber): pause $CPC(recordingTime)"
	set CPC(state) "PAUSE"
	$CPC(stateWidget) configure -fg black
	$CPC(timecodeWidget) configure -fg black
	$w.start configure -image $CPC(playImage) \
   		-command " startRecording $w " 
}

proc stopRecording { w } {
	global CPC

	after cancel $CPC(recordCallback)
	set CPC(recordCallback) {}
	NCast_RecordOff $CPC(g2Sock)
	incr CPC(recordBase) [expr [clock seconds]-$CPC(recordStart)]
	writeLog $CPC(lfd) "$CPC(talkNumber): stop $CPC(recordingTime) ([clock format [clock seconds]])"
	set CPC(state) "STOP"
	$CPC(stateWidget) configure -fg black
	$CPC(timecodeWidget) configure -fg black
	dim $w.start
	dim $w.stop
}

proc resetRecording { w } {
	global CPC

	if {$CPC(state)!="STOP"} {
		tk_messageBox -type ok \
			-message "Cannot reset talk unless G2 status is STOP"
		return
	}
	incr CPC(talkNumber)
	set CPC(recordingTime) "00:00:00"
	set CPC(state) "READY"
	set CPC(recordBase) 0
	set CPC(recordStart) 0
	undim $w.start
	$w.start config -image $CPC(playImage) \
   		-command " startRecording $w " 
	$CPC(stateWidget) configure -fg black
}

# talk number can only be updated when READY or STOP
proc updateSpinbox {} {
	global CPC

	if {($CPC(state)=="READY")||($CPC(state)=="STOP")} {
		return
	}
	incr CPC(talkNumber) -1
	tk_messageBox -type ok -message "Cannot change talk when recording"
}

proc markBeginIntroduction { w } {
	global CPC

	if {$CPC(state) == "READY"} {
		error "Cannot mark time until recording is started"
	} else {
		writeLog $CPC(lfd) "$CPC(talkNumber): intro $CPC(recordingTime) "
		$w config -text "Mark Presentation" \
			-command " markBeginPresentation $w "
	}
}

proc markBeginPresentation { w } {
	global CPC

	writeLog $CPC(lfd) "$CPC(talkNumber) presentation $CPC(recordingTime)"
	$w config -text "Mark Questions" \
		-command " markBeginQuestions $w "
}

proc markBeginQuestions { w } {
	global CPC

	writeLog $CPC(lfd) "$CPC(talkNumber): questions $CPC(recordingTime)"
	dim $w
}

#
# Procedures to access Kramer seamless switcher (VP-720DS)
#
proc openVP {} {
	global CPC

	if {[catch {VP720_Open $CPC(vpDevice)} retval] != 0} {
		tk_messageBox -type ok -message "VP720 could not open device: $retval"
		return
	}
	set CPC(vpCHID) $retval
	VP720_FadeToNonBlack
	after 1000
	VP720_PIP "OFF"
	after 1000
	switchToStageVP
	after 1000
	set CPC(vpOutput) [VP720_GetOutput]
	puts stderr "Kramer openned"
	after 1000
}

proc closeVP {} {
	global CPC

	catch {VP720_Close $CPC(vpCHID)}
	set CPC(vpCHID) ""
}

proc putStateVP {} {
	global CPC
	puts -nonewline "\nBlack=$CPC(vpBlack) PIP=$CPC(vpPip) "
	puts "Source=$CPC(vpSource) LastVideo=$CPC(vpLastVideo)\n"
}

proc selectOutputVP {} {
	global CPC

	VP720_SelectOutput $CPC(vpOutput)
	#puts stderr "selectOutputVP: $CPC(vpOutput) -- not implemented yet"
	update
}

proc selectModeVP {} {
	global CPC

	VP720_SetMode "mode_$CPC(vpMode)"
	#puts stderr "SelectMode: $CPC(vpMode) "
	update
}

proc blackOnVP {aButton} {
	global CPC

	set CPC(vpBlack) 1
	VP720_FadeToBlack
	$aButton configure -text "Fade From Black" -command "blackOffVP $aButton"
	update
}

proc blackOffVP {aButton} {
	global CPC

	set CPC(vpBlack) 0
	VP720_FadeToNonBlack
	$aButton configure -text "Fade To Black" -command "blackOnVP $aButton"
	update
}

proc pipOffVP {aButton} {
	global CPC

	set CPC(vpPip) 0
	VP720_PIP "OFF"
	$aButton configure -text "PIP ON" -command "pipOnVP $aButton"
	update
}

proc pipOnVP {aButton} {
	global CPC

	set CPC(vpPip) 1
	VP720_PIP "ON"
	$aButton configure -text "PIP OFF" -command "pipOffVP $aButton"
	update
}

proc pipSwapVP {} {
	global CPC

	VP720_PIP "SWAP"
	if {$CPC(vpSource)!="VGA"} {
		set CPC(vpSource) "VGA"
	} else {
		set CPC(vpSource) $CPC(vpLastVideo)
	}
}

# Procedures to switch between various video sources
proc switchToStageVP {} {
	global CPC

	if {$CPC(vpSource) != "WIDE"} {
		set CPC(vpSource) "WIDE"
		set CPC(vpLastVideo) "WIDE"
		VP720_SetSource "S-VIDEO1"
	}
}

proc switchToSpeakerVP {} {
	global CPC

	if {$CPC(vpSource) != "CLOSEUP"}  {
		set CPC(vpSource) "CLOSEUP"
		set CPC(vpLastVideo) "CLOSEUP"
		VP720_SetSource "S-VIDEO2"
	}
}

proc switchToGraphicVP {} {
	global CPC

	if {$CPC(vpSource) != "VGA"}  {
		set CPC(vpSource) "VGA"
		VP720_SetSource "VGA"
	}
}

#
# Misc UI functions
#
proc vcc3Launch {} {
	global CPC
	set cmd [list exec  \
		mash c:/confcap/bin/vcc3client -host $CPC(vccServerHost) \
	   		-port $CPC(vccServerPort) -dev $CPC(vccDevice) "&" ]
	#puts stderr "vcc3Launch cmd = \n\t$cmd"
	if {[catch $cmd errcode] != 0} {
			# exec failed
			tk_messageBox -type ok -message "Could not launch vcc3Client ($errcode)"
	} else {
		# vcc3client started
		set CPC(vcc3Server) $errcode
		#puts stderr "vcc3Launch: launched the client ($errcode)"
	}
}

#
# Make GUI for the application
#
proc makeGUI {} {
	global CPC G2

	# setup menubar
	Menu_Setup .menubar
	foreach x {File Edit Tools} {
		Menu $x
	}
	#Menu_Command File "Open G2 Connection" { openG2 }
	#Menu_Command File "Close G2 Connection" { closeG2 }
	Menu_Separator File
	Menu_Command File "Exit" { exitProgram }
	Menu_Check Tools "Show Tk Console" CPC(tkConsole) { toggleTkConsole }
	Menu_Command Tools "Launch VCC3 Controls" { vcc3Launch }

	# setup gui application controls
    frame .gui -relief ridge -borderwidth 2

	frame .gui.talk
	label .gui.talk.label -text "Talk Number: " 
	spinbox .gui.talk.num -textvariable CPC(talkNumber) \
		-from 1 -to 100 -width 4 -command "updateSpinbox" \
		-state readonly -justify center
	pack .gui.talk.label .gui.talk.num -side left
	pack .gui.talk -side top

	# rest of frame made up of top part and bottom part
	frame .gui.top
	set g2 .gui.top		;# G2 connection frame
	set rw .gui.top		;# recording commands frame
	set mw .gui.top		;# marking commands frame

	# g2 connnection commands
	labelframe $g2.state -text "NCast G2"
	label $g2.state.label -text "Status"
	entry $g2.state.entry -textvariable CPC(state) \
		-justify center -state readonly
	set CPC(stateWidget) $g2.state.entry
	pack $g2.state.label $g2.state.entry -side left -padx 5 -pady 5

	# recording commands
	labelframe $rw.record -text "Recording" 
	button $rw.record.start -image $CPC(playImage)  \
		-command " startRecording $rw.record"
	#button $rw.record.pause -text "Pause" \
	#	-command " pauseRecording $rw.record " -state disabled
	button $rw.record.stop -image $CPC(stopImage) \
		-command " stopRecording $rw.record " -state disabled
	pack $rw.record.start $rw.record.stop \
		-side left -padx 5 -pady 5

	Menu_Command Tools "Reset Recording" " resetRecording $rw.record "

	# mark start time commands
	labelframe $mw.mark -text "Recording Time"
	frame $mw.mark.time 
	label $mw.mark.time.label -text "Current Time"
	entry $mw.mark.time.entry -textvariable CPC(recordingTime) \
		-state readonly -bg grey -fg black -justify center
	set CPC(timecodeWidget) $mw.mark.time.entry
	grid $mw.mark.time.label $mw.mark.time.entry
	pack $mw.mark.time -padx 5 -pady 5
	button $mw.mark.button -text "Mark Introduction" \
		-command " markBeginIntroduction $mw.mark.button "
	pack $mw.mark.button -side top -pady 5

	# pack the record and time widgets across the top
	pack $g2.state $rw.record $mw.mark -side left -padx 10 -pady 5

	# switching commands
	labelframe .gui.bottom -text "Production Switching" 
	set psw .gui.bottom	;# switching commands frame
	frame $psw.switch 
	labelframe $psw.switch.c1 -text "Source"
	button $psw.switch.c1.stage -text "Stage" -command { switchToStageVP }
	button $psw.switch.c1.speaker -text "Speaker" -command { switchToSpeakerVP }
	button $psw.switch.c1.graphic -text "Graphic" -command { switchToGraphicVP }
	pack $psw.switch.c1.stage $psw.switch.c1.speaker $psw.switch.c1.graphic \
		-fill x -padx 10 -pady 5 -side top

	labelframe $psw.switch.c2 -text "Effects"
	button $psw.switch.c2.piponoff -text "PIP ON" \
		-command " pipOnVP  $psw.switch.c2.piponoff "
	button $psw.switch.c2.pipswap -text "PIP SWAP" \
		-command { pipSwapVP }
	button $psw.switch.c2.fadetofrom -text "Fade To Black" \
		-command " blackOnVP $psw.switch.c2.fadetofrom "
	pack $psw.switch.c2.piponoff $psw.switch.c2.pipswap \
			$psw.switch.c2.fadetofrom \
		-fill x -padx 10 -pady 5 -side top

	labelframe $psw.switch.c3 -text "VP Controls"
	set i 0
	foreach item {menu status up down select end} {
		button $psw.switch.c3.$item -text [string totitle $item 0 0] \
			-command " VP720_Command $item "
		grid $psw.switch.c3.$item  \
			-row [expr $i % 2] -column [expr $i / 2] \
			-padx 5 -pady 5 -sticky news
		incr i
	}
	$psw.switch.c3.up configure -image $CPC(upArrowImage)
	$psw.switch.c3.down configure -image $CPC(downArrowImage)
	$psw.switch.c3.end configure -image $CPC(leftArrowImage)
	$psw.switch.c3.select configure -image $CPC(rightArrowImage)

	# make mode/output frame mo
	frame $psw.mo
	labelframe $psw.mo.output -text "Output Format"
	set i 0
	foreach item {vga svga xga} {
		radiobutton $psw.mo.output.$i -variable CPC(vpOutput) \
			-text $item -value $item -command { selectOutputVP }
		pack $psw.mo.output.$i -side left
		incr i
	}

	labelframe $psw.mo.mode -text "Display Mode"
	set i 0
	set CPC(vpMode) cinema
	foreach item {normal presentation cinema nature} {
		radiobutton $psw.mo.mode.$i -variable CPC(vpMode) \
			-text $item -value $item -command { selectModeVP }
		pack $psw.mo.mode.$i -side left
		incr i
	}

	# now put these widgets into the interface
	pack $psw.switch.c1 $psw.switch.c2 $psw.switch.c3 \
		-pady 5 -padx 5 -side left -expand true
	pack $psw.switch -side top -padx 20 -pady 5 

	pack $psw.mo.output $psw.mo.mode -pady 5 -padx 5 -side left
	pack $psw.mo -pady 5 -padx 5 -side bottom


	# now pack/grid everything up
	pack .gui.top .gui.bottom -side top -padx 5 -pady 5
	pack .gui -side top
}

proc main {} {
	global CPC

    wm title . "confcap: Conference Capture Production Control"
	update
	openG2 
	openVP 
	makeGUI 
	set logFile "capture.log"
	set CPC(lfd) [openLog $logFile]
	writeLog $CPC(lfd) "\n*** Run started at [clock format [clock seconds]] ***\n"
}

proc exitProgram {} {
	global CPC

	closeG2 
	closeVP
	closeLog $CPC(lfd)
	exit
}

# start application
main
vwait forever
