
# test command - str is just 4 char code
proc ktryit { str } {
	global VP
	puts -nonewline $VP(CHID) "L $str\r"
}

# VP720-DS state variables
set VP(CHID) ""
set VP(Open) -1		;# -1 closed, 0 openning, 1 openned
set VP(Busy) 0
set VP(Output) ""
set VP(DebugVP) 0
set VP(String) ""
set VP(CmdType) ""

proc VP720_Open { device } {
	global VP
	if {[catch {open $device RDWR} retval] != 0} {
		error "Cannot open serial device (open returns \"$retval\")."
	} else {
		set VP(Open) 0
		set VP(CHID) $retval
		fconfigure $VP(CHID) -mode "9600,n,8,1" -blocking 0 \
			-buffering none -translation binary
		fileevent $VP(CHID) readable [list serialRead $VP(CHID)]
		#puts "Serial channel openned and read callback installed."
	}
	after 500
	set VP(Open) 1
	return $VP(CHID)
}

proc serialRead { chan } {
	global VP
	if { [eof $chan] } {
		puts stderr "Closing serial channel $chan"
		catch {close $chan}
		set VP(CHID) ""
		return
	}
	if {!$VP(Busy)||($VP(Open)<1)} {
		set data [read $chan]
		#set size [string length $data]
		#puts "VP not busy, but received $size bytes: $data "
		return
	}
	append VP(String) [string map {\r \x20 \n \x20} [read $chan]]
	if {$VP(DebugVP)} {
		puts stderr "VP720: $VP(String)"
	}
	set VP(String) [regsub -all " " $VP(String) ""]
	if {[string match {[lL]} $VP(CmdType)] 
			&& [regexp "Done>" $VP(String)]} {
		# found the end of an "L" command
		set VP(Busy) 0
		update
	} elseif {[string match {[kK]} $VP(CmdType)] 
				&& [regexp ">" $VP(String)]} {
		# found the end of a "K" command
		set VP(Busy) 0
		update
	}
	#set data [read $chan]
	#set size [string length $data]
	#puts "received $size bytes: $data "
}

proc doVPCommand { str } {
	global VP

	set VP(Busy) 1
	set VP(String) ""
	set VP(CmdType) [string index $str 0]
	puts $VP(CHID) $str
	tkwait variable VP(Busy)
}

proc VP720_Close {} {
	global VP
	catch {close $VP(CHID)}
	set VP(CHID) ""
	return
}

proc VP720_Command { cmd } {
	global VP

	switch -exact -- $cmd {
		"end"	-
		"menu" { doVPCommand "L 12ED" }
		"up" { doVPCommand "L 32CD" }
		"down" { doVPCommand "L 33CC" }
		"select" { doVPCommand "L 36C9" }
	}
}

proc VP720_PIP { arg } {
	global VP

	if {$arg == "ON"} {
		doVPCommand "L B04F"
	} elseif {$arg == "OFF"} {
		doVPCommand "L B14E"
	} elseif {$arg == "SWAP"} {
		doVPCommand "L 2CD3"
	}
}

proc VP720_SetMode { mode } {
	global VP

	switch -exact -- $mode {
		"mode_normal" {
			set str "26D9"
		}
		"mode_presentation" {
			set str "27D8"
		}
		"mode_cinema" {
			set str "28D7"
		}
		"mode_nature" {
			set str "29D6"
		}
		default {
			tk_messageBox -type ok -message "Illegal argument to VP720_SetMode ($mode)\nShould be mode_normal"
		}
	}
	doVPCommand "L $str"
}

proc VP720_SetSource { mode } {
	global VP

	switch -exact -- $mode {
		"VGA" {
			set str "16E9"
		}
		"C-VIDEO1" {
			# c-video1
			set str "39C6"
		}
		"C-VIDEO2" {
			# c-video2
			set str "3AC5"
		}
		"S-VIDEO1" {
			# s-video1
			set str "3BC4"
		}
		"S-VIDEO2" {
			# s-video2
			set str "3CC3"
		}
	}
	doVPCommand "L $str"
}

proc VP720_FadeToBlack {} {
	global VP

	# Blank set ON
	doVPCommand "L B649"
	tkwait variable VP(Busy)
}

proc VP720_FadeToNonBlack {} {
	global VP

	# Blank set OFF
	#puts stderr "Entering VP720_FadeToNonBlack"
	doVPCommand "L B748"
	#puts stderr "Exiting VP720_FadeToNonBlack"
}

proc VP720_SelectOutput { x } {
	global VP

	if {$VP(Output) != $x} {
		switch -exact -- $x {
			"vga" {
				set code 00
			}
			"svga" {
				set code 01
			}
			default -
			"xga" {
				set code 02
			}
		}
		doVPCommand BA45 $code"
		set VP(Output) $x
	}
}

proc VP720_Status {} {
	global VP

	doVPCommand "L 37C8"
}

proc VP720_GetOutput {} {
	global VP

	doVPCommand "K BA45"
	# note: VP(String) has CR/LF/SPACE removed
	switch -regexp -- $VP(String) { 
		".*K00FF" {
			set VP(Output) vga	;# 640x480
		} 
		".*K01FE" {
			set VP(Output) svga	;# 800x600
		}
		".*K02FD" {
			set VP(Output) xga	;# 1024x768
		}
		".*K03FC" {
			set VP(Output) sxga	;# 1280x1024
		}
		".*K04FB" {
			set VP(Output) uxga	;# 1600x1200
		}
		default {
			set VP(Output) $VP(String)	;# unknown
		}
	}
	return $VP(Output)
}
