
#
# Menubar package taken from Welch/Jones book

proc Menu_Setup { menubar } {
	global themenu
	menu $menubar
	# associate menu with main window
	set top [winfo parent $menubar]
	$top config -menu $menubar
	set themenu(menubar) $menubar
	set themenu(uid) 0
}

proc Menu { label } {
	global themenu
	if [info exists themenu(menu,$label)] {
		error "Menu $label already defined"
	}
	# create the cascade menu
	set menuName $themenu(menubar).mb$themenu(uid)
	incr themenu(uid)
	menu $menuName -tearoff 1
	$themenu(menubar) add cascade -label $label -menu $menuName
	# remember the name to menu mapping
	set themenu(menu,$label) $menuName
}

proc MenuGet { menuName } {
	global themenu
	if [catch {set themenu(menu,$menuName)} m] {
		return -code error "No such menu: $menuName"
	}
	return $m
}

proc Menu_Command { menuName label command } {
	set m [MenuGet $menuName]
	$m add command -label $label -command $command
}

proc Menu_Check { menuName label var { command {} } } {
	set m [MenuGet $menuName]
	$m add check -label $label -command $command -variable $var
}

proc Menu_Radio { menuName label var {val {}} {command {}} } {
	set m [MenuGet $menuName]
	if {[string length $val] == 0} {
		set val $label
	}
	$m add radio -label $label -command $command -value $val -variable $var
}

proc Menu_Separator { menuName } {
	[MenuGet $menuName] add separator
}

proc Menu_Cascade { menuName label } {
	global themenu
	set m [MenuGet $menuName]
	if [info exists themenu(menu,$label)] {
		error "Menu $label already defined"
	}
	set sub $m.sub$themenu(uid)
	incr themenu(uid)
	menu $sub -tearoff 0
	$m add cascade -label $label -menu $sub
	set themenu(menu,$label) $sub
}

# test code:
#Menu_Setup .menubar
#Menu File
#Menu_Command File Exit {exit}
#Menu Sampler
#Menu_Command Sampler Hello! {puts "hello world"}
#Menu_Check Sampler Boolean foo {puts "foo = $foo"}
#Menu_Separator Sampler
#Menu_Cascade Sampler Fruit
#Menu_Radio Fruit apple fruit
#Menu_Radio Fruit orange fruit
#Menu_Radio Fruit kiwi fruit
#Menu_Command Sampler WhatIsFruit {puts "fruit=$fruit"}
#
