

#
#  API to control NCast G2
#
set G2(CHID) ""			;# G2 socket
set G2(State) CLOSED	;# State of communication to G2
set G2(Status) ""		;# G2 status string ("!"=> needs update)
set G2(Command) ""		;# Last command sent to G2
set G2(Response) ""		;# Command response, may be null
set G2(Retval) ""		;# Command completed string (i.e., "+/-COMMAND")

set debugG2EventHandling 0

proc debugG2 { msg } {
	global debugG2EventHandling 
	if {$debugG2EventHandling} {
		puts stderr $msg
		flush stderr
	}
}

proc dumpg2 {} {
	global G2
	puts "CHID=$G2(CHID)"
	puts "State=$G2(State)"
	puts "Status=$G2(Status)"
	puts "Command=$G2(Command)"
	puts "Retval=$G2(Response)"
	puts "Response=$G2(Retval)"
}

proc NCast_Open { host port } {
	global G2

	debugG2 "NCast_Open: host=\"$host\" port=\"$port\""
	if {[catch {socket $host $port} G2(CHID)] != 0} {
		error "Cannot open socket (socket returns \"$G2(CHID)\")."
	}
	set G2(Status) "!"
	set G2(State) OPENING
	fconfigure $G2(CHID) -blocking 0 -buffering line
	fileevent $G2(CHID) readable {g2Read}
	puts $G2(CHID) "IdTelnet,002,ncast"
	set afterID [after 30000 {error "Cannot open connection."}]
	while {1} {
		vwait G2(Retval)
		debugG2 "NCast_Open: returned from vwait on G2(Retval)=\"$G2(Retval)\""
		if {[regexp {^[\+\-]IdTelnet} $G2(Retval)]} {
			debugG2 "\t=> recognized as end of command"
			# command completed
			after cancel $afterID
			set G2(State) IDLE
			return $G2(CHID)
		} 
		debugG2  "\t=> not end of command"
	}
}

proc NCast_Close { chID } {
	global G2
	g2Reset
}

proc NCast_LoopbackOn {chID} {
	global G2
	g2SendCmd $chID "L1"
}

proc NCast_LoopbackOff {chID} {
	global G2
	g2SendCmd $chID "L0"
}

proc NCast_RecordOn {chID} {
	global G2
	g2SendCmd $chID "C1"
}

proc NCast_RecordOff {chID} {
	global G2
	g2SendCmd $chID "PE"
}

proc NCast_RecordPause {chID} {
	global G2
	g2SendCmd $chID "R2"
}

proc NCast_RecordContinue {chID} {
	global G2
	g2SendCmd $chID "R3"
}

#
# Utility procedures for communicating with NCast G2
#
proc g2SendCmd { chID cmd } {
	global G2 g_

	if {$chID == ""} {
		return
	}
	if {(![info exists G2(CHID)])||($G2(CHID) != $chID)} {
		tk_messageBox -type ok \
			-message "g2SendCmd: socket not right - is it openned?"
		return
	}
	if {$G2(State) != "IDLE"} {
		tk_messageBox -type ok \
			-message "g2SendCmd: G2 connection is busy (State=$G2(State))"
		return
	}
	set G2(State) "COMMANDPENDING"
	set G2(Retval) ""
	set G2(Response) ""
	puts $chID $cmd
	set afterID [after 30000 {error "Cannot open connection."}]
	while {1} {
		vwait G2(Retval)
		debugG2 "g2SendCmd: returned from vwait on G2(Retval)=\"$G2(Retval)\""
		if {[regexp {^[\+\-]} $G2(Retval)]} {
			debugG2 "\t=> recognized as end of command"
			# command completed
			after cancel $afterID
			set G2(State) "IDLE"
			return $G2(CHID)
		} 
		debugG2 "\t=> not end of command"
	}
}

proc g2Read {} {
	global G2 

    if {[catch { gets $G2(CHID) } line] != 0} {
		tk_messageBox -type ok -message "Error reading G2 socket."
		return
	}
	if {[string length $line] == 0} {
		return
	}
	set ch [string index $line 0]
	if {$ch == "&"} {
		return
	}
	if {$ch == "!"} {
		set G2(Status) "!"
	} elseif {($ch == "+" )||($ch == "-")} {
		set G2(Retval) $line
	} else {
		set G2(Response) $line
	}
}

proc g2Reset { } {
	global G2

	catch {close $G2(CHID)}
	set G2(CHID) ""	
	set G2(State) CLOSED
	set G2(Status) ""	
	set G2(Command) ""
	set G2(Response) ""
	set G2(Retval) ""
}

